package sun.awt.image;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.IOException;
import java.awt.image.ColorModel;
import java.util.Hashtable;
import java.awt.image.ImageConsumer;
import java.io.InputStream;

public abstract class ImageDecoder
{
    InputStreamImageSource source;
    InputStream input;
    Thread feeder;
    protected boolean aborted;
    protected boolean finished;
    ImageConsumerQueue queue;
    ImageDecoder next;
    
    public ImageDecoder(InputStreamImageSource source, InputStream input) {
        this.source = source;
        this.input = input;
        this.feeder = Thread.currentThread();
    }
    
    public boolean isConsumer(ImageConsumer imageConsumer) {
        return ImageConsumerQueue.isConsumer(this.queue, imageConsumer);
    }
    
    public void removeConsumer(ImageConsumer imageConsumer) {
        this.queue = ImageConsumerQueue.removeConsumer(this.queue, imageConsumer, false);
        if (!this.finished && this.queue == null) {
            this.abort();
        }
    }
    
    protected ImageConsumerQueue nextConsumer(ImageConsumerQueue next) {
        synchronized (this.source) {
            if (this.aborted) {
                return null;
            }
            next = ((next == null) ? this.queue : next.next);
            while (next != null) {
                if (next.interested) {
                    return next;
                }
                next = next.next;
            }
        }
        return null;
    }
    
    protected int setDimensions(int n, int n2) {
        ImageConsumerQueue nextConsumer;
        int n3;
        for (nextConsumer = null, n3 = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n3) {
            nextConsumer.consumer.setDimensions(n, n2);
        }
        return n3;
    }
    
    protected int setProperties(Hashtable properties) {
        ImageConsumerQueue nextConsumer;
        int n;
        for (nextConsumer = null, n = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n) {
            nextConsumer.consumer.setProperties(properties);
        }
        return n;
    }
    
    protected int setColorModel(ColorModel colorModel) {
        ImageConsumerQueue nextConsumer;
        int n;
        for (nextConsumer = null, n = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n) {
            nextConsumer.consumer.setColorModel(colorModel);
        }
        return n;
    }
    
    protected int setHints(int hints) {
        ImageConsumerQueue nextConsumer;
        int n;
        for (nextConsumer = null, n = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n) {
            nextConsumer.consumer.setHints(hints);
        }
        return n;
    }
    
    protected void headerComplete() {
        this.feeder.setPriority(3);
    }
    
    protected int setPixels(int n, int n2, int n3, int n4, ColorModel colorModel, byte[] array, int n5, int n6) {
        this.source.latchConsumers(this);
        ImageConsumerQueue nextConsumer;
        int n7;
        for (nextConsumer = null, n7 = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n7) {
            nextConsumer.consumer.setPixels(n, n2, n3, n4, colorModel, array, n5, n6);
        }
        return n7;
    }
    
    protected int setPixels(int n, int n2, int n3, int n4, ColorModel colorModel, int[] array, int n5, int n6) {
        this.source.latchConsumers(this);
        ImageConsumerQueue nextConsumer;
        int n7;
        for (nextConsumer = null, n7 = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n7) {
            nextConsumer.consumer.setPixels(n, n2, n3, n4, colorModel, array, n5, n6);
        }
        return n7;
    }
    
    protected int imageComplete(int n, boolean b) {
        this.source.latchConsumers(this);
        if (b) {
            this.finished = true;
            this.source.doneDecoding(this);
        }
        ImageConsumerQueue nextConsumer;
        int n2;
        for (nextConsumer = null, n2 = 0; (nextConsumer = this.nextConsumer(nextConsumer)) != null; ++n2) {
            nextConsumer.consumer.imageComplete(n);
        }
        return n2;
    }
    
    public abstract void produceImage() throws IOException, ImageFormatException;
    
    public void abort() {
        this.aborted = true;
        this.source.doneDecoding(this);
        this.close();
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                ImageDecoder.this.feeder.interrupt();
                return null;
            }
        });
    }
    
    public synchronized void close() {
        if (this.input != null) {
            try {
                this.input.close();
            }
            catch (IOException ex) {}
        }
    }
}
